/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.ip;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.BitSet;

/**
 * Represents an IP address mask that consists of an IPv4 address and a subnet mask. The subnet mask is a number of bits
 * that defines the network prefix length, i.e. a number between 0 and 32 vor IP4 addresses.
 *
 * @author jdr
 *
 */
public class IP4AddressMask implements IPAddressMask {

    private InetAddress m_address;

    private IP4Mask m_mask;

    /**
     * Creates a new IP4 address mask.
     *
     * @param address
     *            Base IP4 address of the IP address mask.
     * @param mask
     *            subnet mask length to be applied. Must be a value between 0 and 32.
     *
     */
    public IP4AddressMask(final Inet4Address address, int mask)  {

        m_mask = new IP4Mask(mask);

        m_address = address;


    }

    public byte[] getMask() {
        return m_mask.asByteArray();
    }

    public InetAddress getAddress() {
        return m_address;
    }

    /**
     * Mask as a String in the form of "<code>190.10.1.0/24</code>".
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(20);
        sb.append(m_address.getHostAddress()).append('/').append(m_mask.getPrefixLength());
        return sb.toString();
    }

    public boolean contains(InetAddress address) {
        return m_mask.mask(m_address).equals(m_mask.mask(address));
    }

    public InetAddress first() {
        throw new UnsupportedOperationException();
    }

    public InetAddress last() {
        throw new UnsupportedOperationException();
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((m_address == null) ? 0 : m_address.hashCode());
        result = prime * result + ((m_mask == null) ? 0 : m_mask.hashCode());
        return result;
    }

    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        IP4AddressMask other = (IP4AddressMask) obj;
        if (m_address == null) {
            if (other.m_address != null)
                return false;
        } else if (!m_address.equals(other.m_address))
            return false;
        if (m_mask == null) {
            if (other.m_mask != null)
                return false;
        } else if (!m_mask.equals(other.m_mask))
            return false;
        return true;
    }

    public static class IP4Mask {

        public int hashCode() {
            final int prime = 31;
            int result = 1;
            result = prime * result + m_prefixLength;
            return result;
        }



        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (obj == null)
                return false;
            if (getClass() != obj.getClass())
                return false;
            IP4Mask other = (IP4Mask) obj;
            if (m_prefixLength != other.m_prefixLength)
                return false;
            return true;
        }



        private BitSet m_bitSet;
        private final int m_prefixLength;

        /**
         * Creates a subnet mask defined by a subnet mask prefix. The prefix length defines the number of bits that make
         * the prefix, e.g. the prefix length 4 results in a subnet mask represented by the bit string <code>11110000.00000000.00000000.00000000</code>
         *
         * @param prefixLength the prefix length.
         */
        public IP4Mask(int prefixLength) {
            m_prefixLength = prefixLength;
            if (prefixLength < 0 || prefixLength > 32)
                throw new IllegalArgumentException("Subnet mask value of '" + prefixLength + "' not allowed. Must be in [0..32].");

            m_bitSet = new BitSet(32);
            m_bitSet.set(32-prefixLength, 32);
        }



        /**
         *
         * Creates a bit string representation of the mask, which is a String of '1' and '0' characters.
         *
         * @return the String representation.
         */
        public String asBitString() {

            StringBuffer sb = new StringBuffer(32);

            for (int i = m_bitSet.length()-1; i >= 0; i--) {
                sb.append(m_bitSet.get(i)?'1':'0');
            }

            return sb.toString();
        }

        /**
         * Returns a byte[] representation of the mask.
         * The first element of the array represents the highest value of the mask, e.g. for <code>10.0.0.0</code> the element
         * at index 0 has the value '10'.
         *
         *
         * @return the mask as a byte array.
         */
        public byte[] asByteArray() {
            byte[] byteArray = new byte[4];

            for(int i = m_bitSet.nextSetBit(0); i >= 0; i = m_bitSet.nextSetBit(i+1)) {
                //find out which byte of the array is affected, 0,1,2 or 3.
                int arraynum = 3- (i / 8);

                //find out which bit to set
                int setBit = i % 8;

                byteArray[arraynum] = (byte) (byteArray[arraynum] | (1 << setBit));

            }


            return byteArray;

        }

        /**
         * Applies the mask to an IP address.
         *
         * @param address the address that is supposed to be masked.
         * @return the masked IP address.
         */
        InetAddress mask(InetAddress address) {
            byte[] addressToMask = address.getAddress();
            byte[] mask = asByteArray();
            byte[] maskedAddress = new byte[mask.length];
            for (int i = 0; i < mask.length; i++) {
                maskedAddress[i] = (byte) (addressToMask[i] & mask[i]);
            }
            try {
                return InetAddress.getByAddress(maskedAddress);
            } catch (UnknownHostException e) {
                throw new RuntimeException(e);
            }

        }



        public int getPrefixLength() {
            return m_prefixLength;
        }

    }

}
