/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.ip;

import java.net.InetAddress;

/**
 * Represents a contiguous range of IP addresses defined by a lower and an upper boundary.
 *
 * @author jdr
 *
 */
public class IPIntervalRange implements IPRange {

    private final InetAddress m_lowerBoundary;
    private final InetAddress m_upperBoundary;
    private InetAddressComparator m_inetAddressComparator;


    /**
     * Creates an IP range based on two IP addresses defining an interval, which contains the lower and upper boundaries.
     *
     *
     * @param lowerBoundary IP address defining the lower boundary.
     * @param upperBoundary IP address defining the upper boundary.
     */
    public IPIntervalRange(InetAddress lowerBoundary, InetAddress upperBoundary) {

        m_inetAddressComparator = new InetAddressComparator();

        if (m_inetAddressComparator.compare(lowerBoundary, upperBoundary) > 0) {
            throw new IllegalArgumentException("Lower boundary InetAddress " + lowerBoundary.getHostAddress() + " must be smaller than upper boundary address "+ upperBoundary.getHostAddress()+ ".");
        }

        m_lowerBoundary = lowerBoundary;
        m_upperBoundary = upperBoundary;
    }

    /**
     * Creates an IP range based on a single IP address.
     * The interval thus only contains a single IP address.
     *
     * @param singleAddress the single IP address that is contained in this range.
     */
    public IPIntervalRange(InetAddress singleAddress) {
        this(singleAddress, singleAddress);
    }

    public boolean contains(InetAddress address) {
        return m_inetAddressComparator.compare(address, m_lowerBoundary) >= 0 && m_inetAddressComparator.compare(address, m_upperBoundary) <= 0;
    }

    public InetAddress first() {
        return m_lowerBoundary;
    }

    public InetAddress last() {
        return m_upperBoundary;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer(m_lowerBoundary.getHostAddress());

        if (!m_lowerBoundary.equals(m_upperBoundary)) {
            sb.append('-').append(m_upperBoundary.getHostAddress());
        }

        return sb.toString();
    }

    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((m_lowerBoundary == null) ? 0 : m_lowerBoundary.hashCode());
        result = prime * result + ((m_upperBoundary == null) ? 0 : m_upperBoundary.hashCode());
        return result;
    }

    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        IPIntervalRange other = (IPIntervalRange) obj;
        if (m_lowerBoundary == null) {
            if (other.m_lowerBoundary != null)
                return false;
        } else if (!m_lowerBoundary.equals(other.m_lowerBoundary))
            return false;
        if (m_upperBoundary == null) {
            if (other.m_upperBoundary != null)
                return false;
        } else if (!m_upperBoundary.equals(other.m_upperBoundary))
            return false;
        return true;
    }


}
