/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.ip;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Utilities to convert between {@link IPRange} instances and their human-readable string representation.
 * Supported formats (whitespaces are trimmed automatically):
 * <ul>
 * <li>single IP address: <code>172.12.25.84</code>
 * <li>subnet mask: <code>172.12.25.0/24</code>
 * <li>interval: <code>172.12.25.10-172.12.25.20</code>
 * </ul>
 * 
 * @author jdr
 */
public class IPRangeParser {

    /**
     * Converts a string that is either a single IP address, a subnet mask, or an interval of IP adresses.
     * Accepted format/examples, see above.
     * 
     * @param range
     *            a range in one of the formats described above.
     * @return an appropriate IPRange instance, i.e. {@link IPIntervalRange} for single address or interval and
     *         {@link IP4AddressMask} for a subnet mask.
     * @throws IPRangeParseException
     *             if IP address is malformed or doesn't follow the format described above.
     */
    public static IPRange parseRange(String range) throws IPRangeParseException {
        if (range.indexOf(",") != -1)
            throw new IPRangeParseException("Single IP range must not contain ','! Found: " + range);

        if (range.indexOf("/") != -1) {
            return parseIP4AddressMask(range);
        } else if (range.indexOf("-") != -1) {
            return parseIntervalRange(range);
        } else {
            return toRange(parseSingleAddress(range));
        }

    }

    /**
     * Parses an interval of IP addresses, specified as string, where upper and lower bounds are separated with a dash
     * ('-').
     * 
     * @param intervalRange
     *            defninition of interval.
     * @return the IP range interval.
     * @throws IPRangeParseException
     */
    public static IPRange parseIntervalRange(String intervalRange) throws IPRangeParseException {
        String[] addressStrings = intervalRange.split("-");
        if (addressStrings.length != 2)
            throw new IPRangeParseException("IP range must be in formatted like 'address1-address2'. Found: '"
                    + intervalRange + "'");

        InetAddress address1 = parseSingleAddress(addressStrings[0].trim());
        InetAddress address2 = parseSingleAddress(addressStrings[1].trim());

        return toRange(address1, address2);
    }

    /**
     * Parses a IP address mask defining a subnet of IP addresses defined by a base IP address amd a prefix length. See
     * {@link IP4AddressMask} for a detailed description.
     * 
     * @param addressMask
     *            the IP address mask that should be parsed.
     * @return a subnet mask.
     * @throws IPRangeParseException
     */
    public static IPRange parseIP4AddressMask(String addressMask) throws IPRangeParseException {
        String[] maskElems = addressMask.split("/");

        if (maskElems.length != 2)
            throw new IPRangeParseException("IP address mask must be in formatted like 'address/prefix'. Found: '"
                    + addressMask + "'");
        InetAddress address = parseSingleAddress(maskElems[0].trim());

        if (!(address instanceof Inet4Address)) {
            throw new IPRangeParseException("Only IP4 addresses supported. Found: " + address.getHostAddress());
        }

        int prefixLength = Integer.parseInt(maskElems[1].trim());

        return new IP4AddressMask((Inet4Address) address, prefixLength);
    }

    public static IPRange toRange(InetAddress address) {
        return new IPIntervalRange(address);
    }
    public static IPRange toRange(InetAddress lower, InetAddress upper) {
        return new IPIntervalRange(lower, upper);
    }

    /**
     * Convert an IP address specified as a string.
     * 
     * @param address
     *            the IP address.
     * @return the IP address.
     * @throws IPRangeParseException
     */
    public static InetAddress parseSingleAddress(String address) throws IPRangeParseException {
        // we have to use a regex, otherwise sometimes this results in wrong ip lookup,
        // because getByName uses an dns host look up!
        if (address == null) {
            throw new IllegalArgumentException("<address> must not be null!");
        }
        address = address.trim();

        if (address.equals("")) {
            try {
                return InetAddress.getLocalHost();
            } catch (UnknownHostException e) {
                throw new IPRangeParseException(e);
            }
        }

        Matcher m = Pattern.compile("^(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})\\.(\\d{1,3})$").matcher(address);
        if (!m.matches()) {
            throw new IPRangeParseException("address <" + address + "> is not a valid ip address input!");
        }
        try {
            byte[] ibytes = new byte[4];
            ibytes[0] = (byte) Short.parseShort(m.group(1));
            ibytes[1] = (byte) Short.parseShort(m.group(2));
            ibytes[2] = (byte) Short.parseShort(m.group(3));
            ibytes[3] = (byte) Short.parseShort(m.group(4));
            return InetAddress.getByAddress(ibytes);
            // return InetAddress.getByName(address);
        } catch (UnknownHostException e) {
            throw new IPRangeParseException(e);
        }
    }

    /**
     * Parses a list of IP address range specifications.
     * 
     * @param ranges
     *            the string containing a list of IP range specfications.
     * @param delimeter
     *            delimiter that shall be used to identify the single IP range specifications, e.g. <code>","</code>.
     * @return
     * @throws IPRangeParseException
     */
    public static Collection<IPRange> parseRanges(String ranges, String delimeter) throws IPRangeParseException {

        List<IPRange> newRanges = new ArrayList<IPRange>();
        String[] rangesElems = ranges.split(delimeter);
        for (int i = 0; i < rangesElems.length; i++) {
            newRanges.add(parseRange(rangesElems[i]));
        }

        return newRanges;
    }

    /**
     * @param ranges
     *            a list of {@link IPRange} instances.
     * @param delimeter
     *            delimiter that shall be used to separate the different IP ranges.
     * @return single string with each IP range specification in the order as defined in the given list separated by the
     *         given delimiter.
     */
    public static String encodeRangesAsString(Iterable<IPRange> ranges, String delimeter) {
        StringBuilder sb = new StringBuilder();

        for (IPRange range : ranges) {
            sb.append(range.toString());
            sb.append(delimeter);
        }
        if (sb.length() > 0) {
            sb.delete(sb.length() - delimeter.length(), sb.length());
        }

        return sb.toString();
    }
}
