package org.n52.security.common.xml;

import java.util.List;

import javax.xml.namespace.QName;

/**
 * A chunk generated by {@link XMLChunkIterator}
 */
public class XMLChunk {

    final private List<QName> m_xpath;
    final private boolean m_start;
    final private String m_chunk;

    final XMLChunk m_parent;

    public XMLChunk(boolean start, List<QName> xpath, XMLChunk parent) {
        this(start, xpath, "", parent);
    }

    public XMLChunk(boolean start, List<QName> xpath, String chunk, XMLChunk parent) {
        m_start = start;
        m_xpath = xpath;
        m_chunk = chunk;
        m_parent = parent;
    }

    /**
     * Gets the start element of the chunk. Last element in xpath.
     */
    public QName getStartElement() {
        return m_xpath.get(m_xpath.size() - 1);
    }

    /**
     * Gets the xpath as QName instances.
     */
    public List<QName> getXpath() {
        return m_xpath;
    }

    /**
     * Gets the xpath as simple element string, without namespaces.
     *
     * @return a string like: /elementA/elementB/..
     */
    protected String getXPathSimple() {
        return getXPath(false, false);
    }

    /**
     * Gets the xpath as string with namespaces.
     *
     * @return a string like: /{namespaceX}elementA/{namespaceY}elementB/..
     */
    protected String getXPathWithNamespaces() {
        return getXPath(false, true);
    }

    /**
     * simple method building an xpath string from the qnames in the given list.
     *
     * @param currentXPath
     * @param withPrefixes
     *            flag if prefixes included
     * @param withNameSpaces
     *            flag if namespaces included
     * @return a string like /{namespaceX}x:elementA/{namespaceY}y:elementB/..
     */
    private String getXPath(final boolean withPrefixes,
            final boolean withNameSpaces) {
        StringBuilder xpath = new StringBuilder();
        for (QName qName : m_xpath) {
            xpath.append("/");
            if (withNameSpaces) {
                xpath.append("{").append(qName.getNamespaceURI() != null ? qName.getNamespaceURI() : "").append("}");
            }
            if (withPrefixes && qName.getPrefix() != null && qName.getPrefix().length() > 0) {
                xpath.append(qName.getPrefix()).append(":");
            }
            xpath.append(qName.getLocalPart());
        }
        return xpath.toString();
    }

    public boolean isStart() {
        return m_start;
    }
    public String getChunk() {
        return m_chunk;
    }

    public XMLChunk getParent() {
        return m_parent;
    }
    public boolean hasParent() {
        return m_parent != null;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((m_chunk == null) ? 0 : m_chunk.hashCode());
        result = prime * result + (m_start ? 1231 : 1237);
        result = prime * result + ((m_xpath == null) ? 0 : m_xpath.hashCode());
        return result;
    }
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        XMLChunk other = (XMLChunk) obj;
        if (m_chunk == null) {
            if (other.m_chunk != null)
                return false;
        } else if (!m_chunk.equals(other.m_chunk))
            return false;
        if (m_start != other.m_start)
            return false;
        if (m_xpath == null) {
            if (other.m_xpath != null)
                return false;
        } else if (!m_xpath.equals(other.m_xpath))
            return false;
        return true;
    }
    @Override
    public String toString() {
        return "XMLChunk [start=" + m_start + ", xpath=" + m_xpath + ", chunk=" + m_chunk + "]";
    }

}
