/**
 * *****************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 15.07.2008
 * Modified: $Date $
 *****************************************************************************
 */
/* $Id: $ */
package org.n52.security.common.xml;

import java.io.StringReader;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.n52.security.common.util.AbstractObjectPool;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * Encapsulates context informations for xpath expressions.
 * <p/>
 * Use this class to create new XMLPath objectes and combinations of them.
 * <p/>
 * This class wraps the javax.xml.xpath package with performance improvments and
 * simpler useage.
 *
 * @author Marko Reiprecht Created on 15.07.2008
 * @version $Revision: $
 */
public class XMLPathCtx {

    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(XMLPathCtx.class);

    /**
     * change this property to set your prefered xpath engine. This overwrites
     * the default jaxp mechanims.
     * <p/>
     * The default value is
     * 'org.apache.xpath.jaxp.XPathFactoryImpl;com.sun.org.apache.xpath.internal.jaxp.XPathFactoryImpl',
     * this means first check if xalan is in the classpath, if not use the
     * internal sun engine. If both are not present, the default factory lookup
     * mechanism is used {@link XPathFactory#newInstance()}.
     */
    public static String PREFERED_XPATH_FACTORIES
            = "org.apache.xpath.jaxp.XPathFactoryImpl;com.sun.org.apache.xpath.internal.jaxp.XPathFactoryImpl";
    private volatile static XPathPool xPathPool;

    private XPathPool m_xPathPool;
    private Map<String, XPathExpression> m_nameToPreCompiledExpressionMap = new HashMap<String, XPathExpression>();
    private SimpleNamespaceContext m_namespaceContext = new SimpleNamespaceContext();

    // --------------------------- CONSTRUCTORS ---------------------------
    /**
     * Responsible for creation of the default xpath factory. This ignores the
     * default mechanism, because of incopartibility issues with saxon. It first
     * looks for the factory {@link #PREFERED_XPATH_FACTORY} if it exists this
     * factory is created. Otherwise the normal XPathFactory.newInstance is
     * used.
     *
     * @return a xpath factory.
     */
    private static XPathFactory createXPathFactory() {
        String facstr = PREFERED_XPATH_FACTORIES == null ? "" : PREFERED_XPATH_FACTORIES;
        StringTokenizer facs = new StringTokenizer(facstr, ";");
        while (facs.hasMoreTokens()) {
            String nextFac = facs.nextToken();
            try {
                Class facClass = Class.forName(nextFac);
                XPathFactory fac = (XPathFactory) facClass.newInstance();
                if (LOG.isInfoEnabled()) {
                    LOG.info("use class <" + fac.getClass().getName() + "> as xpath factory.");
                }
                return fac;
            } catch (ClassNotFoundException e) {
                if (LOG.isWarnEnabled()) {
                    LOG.warn("class <" + nextFac + "> does not exist, try next.", e);
                }
            } catch (InstantiationException e) {
                if (LOG.isWarnEnabled()) {
                    LOG.warn("instance of class <" + nextFac
                            + "> not createable.", e);
                }
            } catch (IllegalAccessException e) {
                if (LOG.isWarnEnabled()) {
                    LOG.warn("no access to class <" + nextFac + ">.", e);
                }
            }
        }
        if (LOG.isInfoEnabled()) {
            LOG.info("fall back to normal jaxp mechanism.");
        }
        XPathFactory fac = XPathFactory.newInstance();
        if (LOG.isInfoEnabled()) {
            LOG.info("use class <" + fac.getClass().getName() + "> as xpath factory.");
        }
        return fac;
    }

    /**
     * @return the xPathPool
     */
    private static XPathPool getGlobalXPathPool() {
        if (xPathPool == null) {
            xPathPool = new XPathPool(createXPathFactory());
        }
        return xPathPool;
    }

    /**
     * Constructor creates a new instance.
     *
     * @param xPathPool the xPathPool.
     */
    protected XMLPathCtx(final XPathPool xPathPool) {
        m_xPathPool = xPathPool;
    }

    // --------------------- GETTER / SETTER METHODS ---------------------
    /**
     * Method gets the XPathPool.
     *
     * @return XPathPool.
     */
    private XPathPool getXPathPool() {
        return m_xPathPool;
    }

    // -------------------------- OTHER METHODS --------------------------
    /**
     * Method adds prefix to namespace mapping, so the prefix can be used to
     * qualify xpath expressions.
     *
     * @param prefix the prefix.
     * @param namespace the namespace.
     * @return XMLPathCtx.
     */
    public XMLPathCtx addNamespace(final String prefix, final String namespace) {
        m_namespaceContext.add(prefix, namespace);
        return this;
    }

    /**
     * Adds all namespaces from the given context to the internal prefix list.
     *
     * @param ctx an namespace context.
     * @return this.
     */
    public XMLPathCtx addNamespaces(final SimpleNamespaceContext ctx) {
        m_namespaceContext.addAll(ctx);
        return this;
    }

    /**
     * Adds all namespaces from the given Map<prefix,namespace> to the internal
     * prefix list.
     *
     * @param prefixNamespaceMap a map with prefix to namespace mappings.
     * @return this.
     */
    public XMLPathCtx addNamespaces(final Map<String, String> prefixNamespaceMap) {
        m_namespaceContext.addAll(prefixNamespaceMap);
        return this;
    }

    /**
     * Method creates a new XMLPathCtx instance. The instance uses the internaly
     * xpath instance pool.
     *
     * @return XMLPathCtx.
     */
    public static XMLPathCtx createNew() {
        return new XMLPathCtx(getGlobalXPathPool());
    }

    /**
     * Method findIn constructs a new XMLPath starting in a Document source.
     *
     * @param doc the doc.
     * @return XMLPath.
     */
    public XMLPath findIn(final Document doc) {
        return new XMLPath.SourceXMLPath(this, doc);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a Node source.
     *
     * @param node the node.
     * @return XMLPath.
     */
    public XMLPath findIn(final Node node) {
        return new XMLPath.SourceXMLPath(this, node);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a NodeList source.
     *
     * @param nodeList the nodeList.
     * @return XMLPath.
     */
    public XMLPath findIn(final NodeList nodeList) {
        return new XMLPath.SourceXMLPath(this, nodeList);
    }

    /**
     * Method findIn constructs a new XMLPath starting in a InputSource source.
     *
     * @param source the source.
     * @return XMLPath.
     */
    public XMLPath findIn(final InputSource source) {
        return findIn(DOMParser.createNew().parse(source));
    }

    /**
     * Method findIn constructs a new XMLPath starting in a string source.
     *
     * @param source the source.
     * @return XMLPath.
     */
    public XMLPath findIn(final String source) {
        return findIn(DOMParser.createNew().parse(new InputSource(new StringReader(source))));
    }

    /**
     * Method looks up an XPathExpression. A XPathExpression can be precompiled
     * by using the register mehtods.
     *
     * @param xpath the xpath.
     * @return XPathExpression.
     */
    protected XPathExpression lookUp(final String xpath) {
        // first simple native implementation
        XPathExpression expression = (XPathExpression) m_nameToPreCompiledExpressionMap.get(xpath);
        return expression == null ? preCompile(xpath) : expression;
    }

    /**
     * Method pre compiles an xpath expression.
     *
     * @param xpath the xpath.
     * @return XPathExpression.
     */
    private XPathExpression preCompile(final String xpath) {
        XPath path = getXPathPool().acquire();
        try {
            path.setNamespaceContext(m_namespaceContext);
            return path.compile(xpath);
        } catch (XPathExpressionException e) {
            throw new XMLPathException(e);
        } finally {
            getXPathPool().release(path);
        }
    }

    /**
     * Method registers an xpath as precompiled XPathExpression.
     *
     * @param xpath the xpath.
     * @return XMLPathCtx.
     */
    public XMLPathCtx register(final String xpath) {
        return register(xpath, xpath);
    }

    /**
     * Method registers an xpath as precompiled XPathExpression with a special
     * name.
     * <p/>
     * After that you can use the name when ever an xpath is needed.
     *
     * @param name the name.
     * @param xpath the xpath.
     * @return XMLPathCtx.
     */
    public XMLPathCtx register(final String name, final String xpath) {
        m_nameToPreCompiledExpressionMap.put(name, preCompile(xpath));
        return this;
    }

    // -------------------------- INNER CLASSES --------------------------
    /**
     * Class XPathPool.
     *
     * @author Marko Reiprecht Created on 30.05.2008
     * @version $Revision: $
     */
    private static class XPathPool extends AbstractObjectPool<XPath, Void> {

        private final XPathFactory m_xPathFactory;

        /**
         * @param cachedObjLimit
         */
        public XPathPool(final XPathFactory xPathFactory) {
            super(5);
            m_xPathFactory = xPathFactory;
        }

        /**
         * Method acquires a new XPath from the pool.
         *
         * @return XPath.
         */
        public XPath acquire() {
            return this.acquire(null);
        }

        /**
         * Method releases a XPath back in the pool.
         *
         * @param xPath the transformer.
         */
        public void release(final XPath xPath) {
            xPath.reset();
            this.release(xPath, null);
        }

        @Override
        public XPath create(Void key) {
            return m_xPathFactory.newXPath();
        }
    }

}
